# Examine what LLDP is telling us about the local network connection.
module Facter::Util::LLDP

  # Figure out which LLDP implementation we are using
  def self.implementation
    osfamily = Facter.osfamily
    if osfamily == 'FreeBSD'
      return :openlldp
    elsif osfamily == 'Debian'
      return :lldpd
    else
      Facter.debug("unknown LLDP implementation for OS family #{osfamily}")
      return nil
    end
  end

  def self.lldpd
    interface = interface_name = neighbor = nil
    rv = Hash.new
    lldpctl = Facter::Util::Resolution.exec('/usr/sbin/lldpctl')
    return nil if lldpctl.nil?
    lldpctl.each_line do |line|
      line.chomp!
      if line =~ /^Interface:\s+([^,]+), via:\s+([^,]+),/
	interface_name = $1
	interface = rv[interface_name] = Hash.new
	interface[:protocol] = $2
	interface[:neighbors] = Array.new
	neighbor = nil
      elsif interface && line =~ /^\s+Chassis:\s*$/
	neighbor = Hash.new
	neighbor[:interface] = interface_name
        interface[:neighbors].push neighbor
      elsif neighbor && line =~ /^\s+ChassisID:\s+mac ([[:xdigit:]]{2}(:[[:xdigit:]]{2}){5})\s*$/
	neighbor[:chassis_mac] = $1
      elsif neighbor && line =~ /^\s+SysName:\s+(\S+)\s*$/
	neighbor[:name] = $1
      elsif neighbor && line =~ /^\s+MgmtIP:\s+(\S+)\s*$/
	neighbor[:ip] = $1
      elsif neighbor && line =~ /^\s+PortDescr:\s+(\S+)\s*$/
	neighbor[:port_desc] = $1
      elsif neighbor && line =~ /^\s+MFS:\s+(\d+)\s*$/
	neighbor[:mtu] = $1.to_i
      elsif neighbor && line =~ /^\s+VLAN:\s+(\d+),/
	neighbor[:vlan] = $1.to_i
      end
    end
    rv.delete_if {|k,v| !v.has_key?(:neighbors) || v[:neighbors].length == 0}
  end

  def self.openlldp
    interface = interface_name = neighbor = nil
    rv = Hash.new
    lldpneighbors = Facter::Util::Resolution.exec('lldpneighbors')
    return nil if lldpneighbors.nil?
    lldpneighbors.each_line do |line|
      line.chomp!
      if line =~ /^Interface '([^']+)' has \d+ (\S+) Neighbors/
	interface_name = $1
	interface = rv[interface_name] = Hash.new
	interface[:protocol] = $2
	interface[:neighbors] = Array.new
	neighbor = nil
      elsif interface && line =~ /^Neighbor \d+:\s*$/
	neighbor = Hash.new
	neighbor[:interface] = interface_name
        interface[:neighbors].push neighbor
      elsif neighbor && line =~ /^\s+Chassis ID:\s+MAC Address - ([[:xdigit:]]{2}(\s+[[:xdigit:]]{2}){5})\s*$/
	mac = $1
	neighbor[:chassis_mac] = mac.gsub(' ', ':')
      elsif neighbor && line =~ /^\s+System Name:\s+(\S+)\s*$/
	neighbor[:name] = $1
# not available from OpenLLDP
#      elsif line =~ /^\s+MgmtIP:\s+(\S+)\s*$/
#	neighbor[:ip] = $1
      elsif neighbor && line =~ /^\s+Port Description:\s+(\S+)\s*$/
	neighbor[:port_desc] = $1
#      elsif line =~ /^\s+MFS:\s+(\d+)\s*$/
#	neighbor[:mtu] = $1.to_i
#      elsif line =~ /^\s+VLAN:\s+(\d+),/
#	neighbor[:vlan] = $1.to_i
      end
    end
    rv.delete_if {|k,v| v[:neighbors].nil? || v[:neighbors].length == 0}
  end

  def self.instantiate
    unless (impl = self.implementation).nil?
      Facter.debug("LLDP implementation is #{impl.to_s}")
      data = self.send(impl)
      return nil if data.nil?
      i = 0
      counts = Hash.new
      data.each do |interface,info|
	protocol = info[:protocol].downcase
	counts[protocol] = [] unless counts.has_key?(protocol)
	info[:neighbors].each do |neighbor|
	  counts[protocol].push i
	  neighbor.each do |k,v|
	    fact = [protocol, 'neighbor', i.to_s, k.to_s].join('_')
	    Facter.add(fact.intern) do
	      setcode do
		v
	      end
	    end
	  end
	end
	counts.each do |protocol,count|
	  Facter.add("#{protocol}_neighbors".intern) do
	    setcode do
	      count.collect {|x| x.to_s}.join(",")
	    end
	  end
	end
      end
    end
  end
end

Facter::Util::LLDP.instantiate
